package Downloader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import Base.GroovesharkAudioStream;
import Base.GroovesharkClient;
import Base.Utilities;
import Functions.SearchArtist;
import Functions.SearchArtist.SearchArtistResult;

public class JGroovesharkDownloader {

	private static GroovesharkClient client;
	private static String version = "Pre-Alpha v13";
	private static boolean help = false;
	
	private static void exportClientToXml() throws IOException {
		String filename = new File(Utilities.GetAppPath(), "session.tmp").getPath();

		FileOutputStream out = new FileOutputStream(filename);
		ObjectOutputStream obj = new ObjectOutputStream(out);
		
		obj.writeObject(client);
		obj.close();
		out.flush();
		out.close();
	}
	private static void loadClientFromXml() throws Exception {
		try {
			String filename = new File(Utilities.GetAppPath(), "session.tmp").getPath();
			FileInputStream in = new FileInputStream(filename);
			ObjectInputStream obj = new ObjectInputStream(in);
			
			client = (GroovesharkClient)obj.readObject();
			
		} catch(Exception ex) {
			client = new GroovesharkClient(true);
		}
	}
	
	private static void tryConnect() throws Exception {
		if (new File(Utilities.GetAppPath(), "session.tmp").exists()) {
			loadClientFromXml();
			client.reloadGrooveFix();
		} else 
		{
			client = new GroovesharkClient(true);
		}
	}
	
	private static void refreshSession() throws Exception {
		client = new GroovesharkClient(true);
		exportClientToXml();
	}
	
	public static void main(String[] args) {
		try {
			
			//args = new String[] {"/c", "/l", "David Guetta"};
			System.out.println("SciLor's Java grooveshark(tm).com Downloader" + " " + version);

			System.out.println("Arguments: ");
			for (int i=0; i<args.length; i++) {
				System.out.print("\"" + args[i] + "\"" + " ");
			}
			System.out.println("");
			System.out.println("");
			if (args.length > 1 && args[0].toLowerCase().equals("/c")) {
				tryConnect();
				if (args[args.length - 1].equals("/r")) {
					refreshSession();
					String[] rearg = new String[args.length - 1];
					for (int i=0; i<rearg.length; i++) {
						rearg[i] = args[i];
					}
					args = rearg;
				}
				
				if (args.length == 3) {
					if (args[1].toLowerCase().equals("/b")) {
						downloadSongsFromFile(args[2]);
					} else if (args[1].toLowerCase().equals("/l")) {
						listSearch(args[2]);
					} else { printHelp(); }
				} else if(args.length == 4) {
					if (args[1].toLowerCase().equals("/d")) {
						int x = Integer.valueOf(args[3]).intValue();
						if (x > 0 ) {
							downloadSongFromSearch(args[2], x - 1);
						} else { printHelp(); }
					} else { printHelp(); }
				} else { printHelp(); }
			} else { printHelp(); }
			if (!help) {
				exportClientToXml();
			}
			
			//SearchArtist.SearchArtistResult[] results = client.SearchArtist("Rammstein Bck").result.result;
			
			/*
			System.out.println("Results:");
			for (int i=0; i < results.length; i++) {
				System.out.println(results[i].ArtistName + " - " + results[i].SongName);
			}
			*/
			
			
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("Error occured: ");
			System.err.println(e.getMessage());
			try {
				refreshSession();
			} catch (Exception e2) {
				e2.printStackTrace();
				System.err.println("Error occured(refreshSession): ");
				System.err.println(e.getMessage());
			}
		}
	}
	
	private static void printHelp() {
		System.out.println("Usage: ");
		System.out.println("SciLorsJGroovesharkDownloader.jar [/c] [[/d search x] | [/l search] | [/b filename]] [/r]");
		System.out.println("");
		System.out.println("\t/c\tActivates command line mode.");
		System.out.println("\t/d\tDownloads song [x] from your [search]");
		System.out.println("\t/l\tList songs from your [search]");
		System.out.println("\t/b\tBatch downloads the first result over every search specified in a file line by line.");
		System.out.println("\t/r\tRefresh the cached session.");
		help = true;
	}
	
	private static void listSearch(String search) throws Exception {
		SearchArtistResult[] results = client.SearchArtist(search).result.result;
		if (results.length == 0 ) {
			System.out.println("No results for " + "\"" + search + "\"");
		} else {
			for (int i = 0; i < results.length; i++) {
				System.out.println((i + 1) + ". " + results[i].ArtistName + " - " + results[i].Name + " - " + results[i].AlbumName );
			}
		}
	}
	
	private static void downloadSongsFromFile(String filename) throws Exception {
        if (!new File(filename).exists())
        {
            throw new Exception("Missing file: \"" + filename + "\"");
        }
	    String path = new File(filename).getPath();
	    FileReader input = new FileReader(path);
	    BufferedReader reader = new BufferedReader(input);
	    
	    String line;
	    do {
	    	line = reader.readLine();
	    	try {
	    		downloadSongFromSearch(line, 0);
	    	} catch (Exception ex) {
	    		System.err.println("Broken song? (" + line + ") " + ex.getMessage());
	    	}
	    } while (line != null);
	    
	    reader.close();
	}
	
	private static void downloadSongFromSearch(String search, int id) throws Exception {
		SearchArtist.SearchArtistResult[] results = client.SearchArtist(search).result.result;
		if (results.length > id) {
			downloadSong(results[id]);
		} else {
			if (results.length == 0 ) {
				System.out.println("No results for " + "\"" + search + "\"");
			} else {
				System.out.println("Your id" + "(" + (id + 1) + ")" + " is greater then the count of results" + "(" + results.length + ")");
			}
		}
	}
	
	private static String fixFilename(String filename) {
		String tmpString = "";
		
		for (int i=0; i<filename.length(); i++) {
			try {
				new File(filename.charAt(i) + "").getCanonicalFile();
				if (filename.charAt(i) != '/' && filename.charAt(i) != '\\') {
					tmpString +=  filename.charAt(i);
				}
			} catch(Exception ex) { }
		}
		
		return tmpString.replace("\"", "");
	}
	
	private static void downloadSong(SearchArtistResult song) throws Exception {
		System.out.println("Download: " + song.ArtistName + " - " + song.Name);
		GroovesharkAudioStream stream = client.GetMusicStream(song.SongID);
		String filename = fixFilename(song.ArtistName + " - " + song.Name + " - " + song.AlbumName) + ".mp3";
		
		FileOutputStream writer = new FileOutputStream(filename);
		int readBytes = 0;
		int pos=0;
		int percentage = 0;
		int prevPercentage = 0;
		do {
			byte[] buffer = new byte[4096];
			readBytes = stream.Stream().read(buffer);
			pos += readBytes;
			if (readBytes > 0) writer.write(buffer, 0, readBytes);
			percentage = 100 * pos / (stream.Length() - 1);
			if (percentage > prevPercentage + 4) {
				System.out.println(percentage + "%" + " \"" + filename + "\"");
				prevPercentage = percentage;
			}
		} while (readBytes > 0);
		stream.MarkSongAsDownloaded();

		writer.flush();
		writer.close();
		stream.Stream().close();
	}

}
